package cn.iocoder.yudao.module.depository.dal.mysql.warehouse;


import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.vo.warehouse.WarehouseLineBaseVO;
import cn.iocoder.yudao.module.depository.vo.warehouse.WarehouseLineSearchVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.depository.vo.warehouseLine.WarehouseLineBackVO;
import cn.iocoder.yudao.module.depository.vo.warehouseLine.WarehouseLineQueryVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 仓库线路关联 Mapper
 *
 * @author wlh
 */
@Mapper
public interface WarehouseLineMapper extends BaseMapperX<WarehouseLineDO> {


    default List<WarehouseLineDO> selectList(WarehouseLineBaseVO reqVO) {
        return selectList(new LambdaQueryWrapperX<WarehouseLineDO>()
                .eqIfPresent(WarehouseLineDO::getStartWarehouseId, reqVO.getStartWarehouseId())
                .eqIfPresent(WarehouseLineDO::getDestWarehouseId, reqVO.getDestWarehouseId())
                .eqIfPresent(WarehouseLineDO::getTransportType, reqVO.getTransportType())
                .eqIfPresent(WarehouseLineDO::getStatus, reqVO.getStatus())
                .orderByDesc(WarehouseLineDO::getId));
    }


    default PageResult<WarehouseLineDO> selectPage(PageVO page, Object object) {
        if (object instanceof WarehouseLineQueryVO) {
            WarehouseLineQueryVO vo = (WarehouseLineQueryVO) object;
            return selectPage(page, new LambdaQuery<WarehouseLineDO>()
                    .inIfPresent(WarehouseLineDO::getStartWarehouseId, vo.getStartWarehouseIdList())
                    .inIfPresent(WarehouseLineDO::getDestWarehouseId, vo.getDestWarehouseIdList())
                    .eqIfPresent(WarehouseLineDO::getTransportType, vo.getTransportType())
                    .eqIfPresent(WarehouseLineDO::getStatus, vo.getStatus())
                    .betweenIfPresent(WarehouseLineDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .orderByDesc(WarehouseLineDO::getId));
        }

        return null;
    }

    List<WarehouseLineDO> openedRouterList(@Param("params") WarehouseLineSearchVO params);

    List<WarehouseLineDO> selectBylineIdList(@Param("lineIdList") List<Long> lineIdList);

    @ResultType(LogisticsInfoDto.class)
    @Select({
            "SELECT",
            "ewl.id as line_id,",
            "ewl.transport_type as transport_id,",
            "ewl.start_warehouse_id as start_warehouse_id,",
            "ewl.dest_warehouse_id as dest_warehouse_id,",
            "ew_start.shi as start_city_id,",
            "ew_start.guojia as start_country_id,",
            "ew_start.title_zh as start_title_zh,",
            "ew_start.title_en as start_title_en,",
            "ew_start.qianzhui as start_qianzhui,",
            "ew_start_country.title_zh as start_country_title_zh,",
            "ew_start_country.title_en as start_country_title_en,",
            "ew_start.volume as start_volume,",
            "ew_start.rent_free_days as start_rent_free_days,",
            "ew_start.lock_recipient_days as start_lock_recipient_days,",
            "ew_start.address_zh as start_address_zh,",
            "ew_start.address_en as start_address_en,",
            "ew_start.head as start_head,",
            "ew_start.tell as start_tell,",
            "ew_start.kyc_status as kycStatus,",
            "ew_dest.shi as dest_city_id,",
            "ew_dest.guojia as dest_country_id,",
            "ew_dest.title_zh as dest_title_zh,",
            "ew_dest.title_en as dest_title_en,",
            "ew_dest.qianzhui as dest_qianzhui,",
            "ew_dest_country.title_zh as dest_country_title_zh,",
            "ew_dest_country.title_en as dest_country_title_en,",
            "ew_dest_city.title_zh as dest_city_title_zh,",
            "ew_dest_city.title_en as dest_city_title_en,",
            "ew_dest.volume as dest_volume,",
            "ew_dest.rent_free_days as dest_rent_free_days,",
            "ew_dest.lock_recipient_days as dest_lock_recipient_days,",
            "ew_dest.address_zh as dest_address_zh,",
            "ew_dest.address_en as dest_address_en,",
            "ew_dest.head as dest_head,",
            "ew_dest.tell as dest_tell",
            "FROM",
            "ecw_warehouse_line ewl",
            "JOIN ecw_warehouse ew_start",
            "ON ewl.start_warehouse_id = ew_start.id",
            "JOIN ecw_region ew_start_country",
            "ON ew_start_country.id = ew_start.guojia",
            "JOIN ecw_warehouse ew_dest",
            "ON ewl.dest_warehouse_id = ew_dest.id",
            "JOIN ecw_region ew_dest_country",
            "ON ew_dest_country.id = ew_dest.guojia",
            "JOIN ecw_region ew_dest_city",
            "ON ew_dest_city.id = ew_dest.shi",
            "WHERE ",
            " ewl.id = #{lineId}"
    })
    LogisticsInfoDto getStartInfoAndDestInfoByLineId(@Param("lineId") Long lineId);


    @ResultType(LogisticsInfoDto.class)
    @Select({
            "<script>",
            "SELECT",
            "ewl.id as line_id,",
            "ewl.transport_type as transport_id,",
            "ewl.start_warehouse_id as start_warehouse_id,",
            "ewl.dest_warehouse_id as dest_warehouse_id,",
            "ew_start.shi as start_city_id,",
            "ew_start.guojia as start_country_id,",
            "ew_start.title_zh as start_title_zh,",
            "ew_start.title_en as start_title_en,",
            "ew_start.volume as start_volume,",
            "ew_start.address_zh as start_address_zh,",
            "ew_start.address_en as start_address_en,",
            "ew_start.head as start_head,",
            "ew_start.tell as start_tell,",
            "ew_dest.shi as dest_city_id,",
            "ew_dest.guojia as dest_country_id,",
            "ew_dest.title_zh as dest_title_zh,",
            "ew_dest.title_en as dest_title_en,",
            "ew_dest.volume as dest_volume,",
            "ew_dest.address_zh as dest_address_zh,",
            "ew_dest.address_en as dest_address_en,",
            "ew_dest.head as dest_head,",
            "ew_dest.tell as dest_tell",
            "FROM",
            "ecw_warehouse_line ewl",
            "JOIN ecw_warehouse ew_start",
            "ON ewl.start_warehouse_id = ew_start.id",
            "JOIN ecw_warehouse ew_dest",
            "ON ewl.dest_warehouse_id = ew_dest.id",
            "WHERE ",
            "ewl.start_warehouse_id in ",
            "<foreach item='start' index='index' collection='startWarehouseList' open='(' separator=',' close=')'>",
            "#{start.id}",
            "</foreach>",
            "AND ewl.dest_warehouse_id in ",
            "<foreach item='dest' index='index' collection='destWarehouseList' open='(' separator=',' close=')'>",
            "#{dest.id}",
            "</foreach>",
            "and ewl.transport_type in",
            "<foreach item='transportType' index='index' collection='transportTypeList' open='(' separator=',' close=')'>",
            "#{transportType}",
            "</foreach>",
            "order by ewl.id desc",
            "</script>"
    })
    List<LogisticsInfoDto> getStartInfoAndDestInfo(@Param("startWarehouseList") List<WarehouseDO> startWarehouseList, @Param("destWarehouseList") List<WarehouseDO> destWarehouseList,
                                                   @Param("transportTypeList") List<Integer> transportTypeList);

    @Select("select * from ecw_warehouse_line where start_warehouse_id = #{startWarehouseId} and dest_warehouse_id = #{destWarehouseId} and transport_type = #{transportType} limit 1")
    WarehouseLineDO findOne(@Param("startWarehouseId") Long startWarehouseId, @Param("destWarehouseId") Long destWarehouseId, @Param("transportType") String transportType);

    @Update("update ecw_warehouse_line set deleted = #{deleted} where id = #{id}")
    void updateDeletedById(@Param("id") Long id, @Param("deleted") Boolean deleted);

    @Select("select * from ecw_warehouse_line where id = #{id}")
    WarehouseLineDO findOneIgnoreDeleted(@Param("id") Long id);

    IPage<WarehouseLineBackVO> getWarehouseLinePage(IPage<WarehouseLineBackVO> page, @Param("query") WarehouseLineQueryVO query);

    /**
     * 根据线路id获取目的仓一致的全部线路id
     *
     * @param lineId 线路id
     * @return
     */
    @Select("select w1.id from ecw_warehouse_line w1 left join ecw_warehouse_line w2 on w1.dest_warehouse_id = w2.dest_warehouse_id where w1.deleted = 0 and w2.id = #{lineId}")
    List<Long> getDestAsLinesByLineId(@Param("lineId") Long lineId);
}
